@@ -284,7 +284,7 @@ class BundleFormatImporter_1_0(IBundleFormatImporter):
                     microseconds   BIGINT,
                     filename       TEXT,
                     result         SMALLINT,
-                    measurement    NUMERIC(20,10),
+                    measurement    CHARACTER VARYING(512),
                     message        TEXT,
                     test_case_id   TEXT,
                     lineno         INTEGER
@@ -827,6 +827,38 @@ class BundleFormatImporter_1_7(BundleFormatImporter_1_6):
             s_test_run.sources.add(s_source)
 
 
+class BundleFormatImporter_1_7_1(BundleFormatImporter_1_7):
+    """
+    IFormatImporter subclass capable of loading "Dashboard Bundle Format 1.7.1"
+    """
+
+    def _import_sources(self, c_test_run, s_test_run):
+        """
+        Import TestRun.sources
+        """
+        from dashboard_app.models import SoftwareSource
+
+        for c_source in self._get_sw_context(c_test_run).get("sources", []):
+            s_source, source_created = SoftwareSource.objects.get_or_create(
+                project_name=c_source["project_name"],  # required by schema
+                branch_url=c_source["branch_url"],  # required by schema
+                branch_vcs=c_source["branch_vcs"],  # required by schema
+                # required by schema, may be either int or string so upconvert to string
+                branch_revision=str(c_source["branch_revision"]),
+                # optional
+                commit_timestamp=(
+                    datetime_extension.from_json(
+                        c_source["commit_timestamp"])
+                    if "commit_timestamp" in c_source
+                    else None),
+                default_params=c_source.get("default_params", []),
+                test_params=c_source.get("test_params", [])
+            )
+            if source_created:
+                s_source.save()
+            s_test_run.sources.add(s_source)
+
+
 class BundleDeserializer(object):
     """
     Helper class for de-serializing JSON bundle content into database models
@@ -842,6 +874,7 @@ class BundleDeserializer(object):
         "Dashboard Bundle Format 1.5": BundleFormatImporter_1_5,
         "Dashboard Bundle Format 1.6": BundleFormatImporter_1_6,
         "Dashboard Bundle Format 1.7": BundleFormatImporter_1_7,
+        "Dashboard Bundle Format 1.7.1": BundleFormatImporter_1_7_1,
     }
 
     def deserialize(self, s_bundle, prefer_evolution):
@@ -869,14 +902,15 @@ class BundleDeserializer(object):
         """
         assert s_bundle.is_deserialized is False
         s_bundle.content.open('rb')
+        logger = logging.getLogger(__name__)
         try:
-            logging.debug("Loading document")
+            logger.debug("Loading document")
             fmt, doc = DocumentIO.load(s_bundle.content)
-            logging.debug("Document loaded")
+            logger.debug("Document loaded")
             if prefer_evolution:
-                logging.debug("Evolving document")
+                logger.debug("Evolving document")
                 DocumentEvolution.evolve_document(doc)
-                logging.debug("Document evolution complete")
+                logger.debug("Document evolution complete")
                 fmt = doc["format"]
         finally:
             s_bundle.content.close()
@@ -884,9 +918,9 @@ class BundleDeserializer(object):
         if importer is None:
             raise DocumentFormatError(fmt)
         try:
-            logging.debug("Importing document")
+            logger.debug("Importing document")
             importer().import_document(s_bundle, doc)
-            logging.debug("Document import complete")
+            logger.debug("Document import complete")
         except Exception as exc:
-            logging.debug("Exception while importing document: %r", exc)
+            logger.debug("Exception while importing document: %r", exc)
             raise
