@@ -7,8 +7,9 @@ $(document).ready(function () {
 
     ImageReport.prototype.start = function() {
         // Add charts.
-        for (chart_id in this.chart_data) {
-            chart = new ImageChart(chart_id, this.chart_data[chart_id]);
+        for (index in this.chart_data) {
+            chart = new ImageChart(this.chart_data[index].id,
+                                   this.chart_data[index]);
             chart.add_chart();
             this.charts.push(chart);
         }
@@ -68,6 +69,17 @@ $(document).ready(function () {
                 '<div class="legend" id="legend_container_' +
                     this.chart_id + '"></div>');
 
+            // Set chart height.
+            $("#inner_container_" + this.chart_id).height(
+                this.chart_data["chart_height"]
+            );
+            // Set chart legend height.
+            $("#legend_container_" + this.chart_id).height(
+                this.chart_data["chart_height"]
+            );
+
+            // Determine whether dates/build numbers/attributes are numeric .
+            this.is_keys_numeric();
             // Add headline and description.
             this.update_headline();
             // Add filter links.
@@ -82,9 +94,40 @@ $(document).ready(function () {
             this.update_urls();
             // Update events.
             this.update_events();
+            // Update chart/table visibility.
+            this.update_visibility();
         }
     }
 
+    ImageChart.prototype.is_keys_numeric = function() {
+
+        keys = this.get_keys();
+        // Check if all range items are numeric and set the value in chart_data.
+        this.chart_data.is_keys_numeric = is_array_numeric(keys);
+
+        this.chart_data.is_keys_decimal = false;
+        if (this.chart_data.is_keys_numeric) {
+            this.chart_data.is_keys_decimal = is_any_element_decimal(keys);
+        }
+    }
+
+    ImageChart.prototype.get_keys = function() {
+
+        // Get all keys based on chart options.
+        keys = [];
+        for (iter in this.chart_data.test_data) {
+            if (this.chart_data.xaxis_attribute) {
+                item = this.chart_data.test_data[iter]["attribute"];
+            } else {
+                item = this.chart_data.test_data[iter]["number"].split('.')[0];
+            }
+            if (item && keys.indexOf(item) == -1) {
+                keys.push(item);
+            }
+        }
+
+        return keys;
+    }
 
     ImageChart.prototype.update_alias = function() {
         if (Object.keys(this.chart_data.filters).length == 1) {
@@ -192,17 +235,6 @@ $(document).ready(function () {
         $("#filter_links_dialog_" + this.chart_id).dialog("option",
                                                           "height", height );
 
-        // Add percentages if chart type is pass/fail.
-        if (this.chart_data["chart_type"] == "pass/fail") {
-            $("#filter_links_container_" + this.chart_id).append(
-                '<span class="toggle-percentage"><label for="is_percentage_' +
-                    this.chart_id +
-                    '">Toggle percentage</label></span>');
-            $("#filter_links_container_" + this.chart_id).append(
-                '<span class="toggle-checkbox"><input type="checkbox" ' +
-                    'id="is_percentage_' + this.chart_id + '" /></span>');
-        }
-
         // Add legend toggle checkbox.
         $("#filter_links_container_" + this.chart_id).append(
             '<span class="toggle-legend"><label for="is_legend_visible_' +
@@ -212,6 +244,15 @@ $(document).ready(function () {
                 'id="is_legend_visible_' + this.chart_id +
                 '" checked="checked"/></span>');
 
+        // Add delta reporting checkbox.
+        $("#filter_links_container_" + this.chart_id).append(
+            '<span class="toggle-delta"><label for="is_delta_' +
+                this.chart_id + '">Delta reporting</label></span>');
+        $("#filter_links_container_" + this.chart_id).append(
+            '<span class="toggle-checkbox"><input type="checkbox" ' +
+                'id="is_delta_' + this.chart_id +
+                '" checked="checked"/></span>');
+
         $("#filter_links_container_" + this.chart_id).append(
             '<span class="chart-save-img">' +
                 '<a id="chart_menu_' + this.chart_id + '"' +
@@ -236,13 +277,13 @@ $(document).ready(function () {
 
         // Add dates(build number) fields and toggle legend checkbox.
         $("#dates_container_" + this.chart_id).append(
-            '<span style="margin-left: 30px;">Start build number:&nbsp;' +
+            '<span style="margin-left: 30px;">Start:&nbsp;' +
                 '&nbsp;</span>');
         $("#dates_container_" + this.chart_id).append(
             '<span><select id="start_date_' + this.chart_id +
                 '"></select></span>');
         $("#dates_container_" + this.chart_id).append(
-            '<span>&nbsp;&nbsp;&nbsp;&nbsp;End build number:&nbsp;&nbsp;' +
+            '<span>&nbsp;&nbsp;&nbsp;&nbsp;End:&nbsp;&nbsp;' +
                 '</span>');
         $("#dates_container_" + this.chart_id).append(
             '<span><select id="end_date_' + this.chart_id + '"></select>' +
@@ -258,12 +299,8 @@ $(document).ready(function () {
         this.add_settings_events();
     }
 
-    ImageChart.prototype.update_data_tables = function() {
-
-        // Add dialog.
-        $("#main_container").append('<div id="data_table_dialog_' +
-                                    this.chart_id + '"></div>');
 
+    ImageChart.prototype.result_table_as_dialog = function() {
         // Init dialog.
         $('#data_table_dialog_' + this.chart_id).dialog({
             autoOpen: false,
@@ -274,15 +311,35 @@ $(document).ready(function () {
             modal: true,
             resizable: false,
             open: function (event, ui) {
-                $('#data_table_dialog_' + this.chart_id).css('overflow',
-                                                             'hidden');
-
                 $('.scroller').each(function() {
                     $(this).scrollLeft($(this)[0].scrollWidth);
                 });
             }
         });
 
+    }
+
+    ImageChart.prototype.update_visibility = function() {
+        if (this.chart_data.chart_visibility == 'table') {
+            $("#outer_container_" + this.chart_id).hide();
+            $("#table_link_container_" + this.chart_id).hide();
+            $("#filter_links_container_" + this.chart_id).hide();
+            $("#dates_container_" + this.chart_id).hide();
+        } else if (this.chart_data.chart_visibility == 'both') {
+            $("#table_link_container_" + this.chart_id).hide();
+        }
+    }
+
+    ImageChart.prototype.update_data_tables = function() {
+
+        // Add dialog.
+        $("#chart_container_" + this.chart_id).append(
+            '<div id="data_table_dialog_' + this.chart_id + '"></div>');
+
+        if (this.chart_data.chart_visibility == 'chart') {
+            this.result_table_as_dialog();
+        }
+
         // Add skeleton data to dialog.
         $("#data_table_dialog_" + this.chart_id).append(
             '<table id="outer-table"><tr><td>' +
@@ -306,6 +363,12 @@ $(document).ready(function () {
                 this.chart_id +
                 '" href="javascript:void(0)">Results table</a>');
 
+        $('#data_table_dialog_' + this.chart_id).css('overflow', 'hidden');
+
+        $('.scroller').each(function() {
+            $(this).scrollLeft($(this)[0].scrollWidth);
+        });
+
         var chart = this;
         $("#data_table_link_" + this.chart_id).click(function(){
             $('#data_table_dialog_' + chart.chart_id).dialog("open");
@@ -356,14 +419,18 @@ $(document).ready(function () {
                 table[number][test_data["test_filter_id"]] = [];
             }
 
+            measurement = parseFloat(test_data["measurement"]);
+            if (isNaN(measurement)) {
+                measurement = test_data["measurement"];
+            }
             table[number][test_data["test_filter_id"]].push({
                 "passes": test_data["passes"],
                 "pass": test_data["pass"],
                 "skip": test_data["skip"],
                 "total": test_data["total"],
-                "measurement": test_data["measurement"],
+                "measurement": measurement,
                 "attr_value": test_data["attr_value"],
-                "link": test_data["link"],
+                "link": test_data["link"].replace("\\\\\\", ""),
                 "test_run_uuid": test_data["test_run_uuid"],
                 "bug_links": test_data["bug_links"]
             });
@@ -428,9 +495,9 @@ $(document).ready(function () {
                     uuid = cell["test_run_uuid"];
                     relative_index_str = "";
                     if (cell["measurement"]) {
-                        arr = cell["link"].split("/")
-                        relative_index = arr[arr.length-2]
-                        relative_index_str = 'data-relative_index="' + relative_index +'"'
+                        arr = cell["link"].split("/");
+                        relative_index = arr[arr.length-2];
+                        relative_index_str = 'data-relative_index="' + relative_index +'"';
                     }
 
                     table_body += '<td class="' + cls + '" data-chart-id="' +
@@ -441,9 +508,16 @@ $(document).ready(function () {
                             cell["link"] + '">' + cell["passes"] + '/' +
                             cell["total"] + '</a>';
                     } else if (this.chart_data["chart_type"] == "measurement") {
+                        if (isNumeric(cell["measurement"])) {
+                            cell["measurement"] = cell["measurement"].toFixed(2);
+                        }
                         table_body += '<a target="_blank" href="' +
-                            cell["link"] + '">' + cell["measurement"] + '</a>';
+                            cell["link"] + '">' +
+                            cell["measurement"] + '</a>';
                     } else if (this.chart_data["chart_type"] == "attributes") {
+                        if (isNumeric(cell["attr_value"])) {
+                            cell["attr_value"] = cell["attr_value"].toFixed(2);
+                        }
                         table_body += '<a target="_blank" href="' +
                             cell["link"] + '">' + cell["attr_value"] + '</a>';
                     }
@@ -456,8 +530,8 @@ $(document).ready(function () {
 
                     for (bug_link in cell["bug_links"]) {
                         bug = cell["bug_links"];
-                        table_body += '<li class="bug-link">' + bug[bug_link] +
-                            '</li>';
+                        table_body += '<li class="bug-link">' +
+                            bug[bug_link].replace(/\\\\\\/g, "") + '</li>';
                     }
 
                     table_body += '</span>';
@@ -480,29 +554,23 @@ $(document).ready(function () {
     }
 
     ImageChart.prototype.set_dates = function() {
-        // Populate date dropdowns.
-        dates = [];
-        for (iter in this.chart_data.test_data) {
-            item = this.chart_data.test_data[iter]["number"].split('.')[0];
-            if (dates.indexOf(item) == -1) {
-                dates.push(item);
-            }
-        }
+        // Populate range dropdowns.
+        keys = this.get_keys();
 
-        if (this.chart_data.has_build_numbers) {
-            dates.sort(function(x,y) {return x-y;});
+        if (this.chart_data.is_keys_numeric) {
+            keys.sort(function(x,y) {return x-y;});
         } else {
-            dates.sort();
+            keys.sort();
         }
 
-        for (i in dates) {
+        for (i in keys) {
             $("#start_date_" + this.chart_id).append($("<option/>", {
-                value: dates[i],
-                text: dates[i]
+                value: keys[i],
+                text: keys[i]
             }));
             $("#end_date_" + this.chart_id).append($("<option/>", {
-                value: dates[i],
-                text: dates[i]
+                value: keys[i],
+                text: keys[i]
             }));
         }
         $("#end_date_" + this.chart_id + " option:last").attr("selected",
@@ -510,14 +578,16 @@ $(document).ready(function () {
     }
 
     ImageChart.prototype.validate_build_number_selection = function() {
+        // Validates start date field value against end data field value.
+        // start date needs to be "less" then end date.
 
         start_number = $("#start_date_" + this.chart_id).val();
         if (isNumeric(start_number)) {
-	    start_number = parseInt(start_number);
+	    start_number = parseFloat(start_number);
         }
         end_number = $("#end_date_" + this.chart_id).val();
         if (isNumeric(end_number)) {
-	    end_number = parseInt(end_number);
+	    end_number = parseFloat(end_number);
         }
 
         if (start_number >= end_number) {
@@ -565,17 +635,11 @@ $(document).ready(function () {
             chart.update_settings();
         });
 
-        $("#is_percentage_"+this.chart_id).change(function() {
+        $("#is_delta_"+this.chart_id).change(function() {
             chart.update_plot();
             chart.update_settings();
         });
 
-        if (this.chart_data["chart_type"] == "pass/fail") {
-            $("#is_percentage_"+this.chart_id).change(function() {
-                chart.update_plot();
-            });
-        }
-
         $("#has_subscription_link_"+this.chart_id).click(function() {
             $("#has_subscription_" + chart.chart_id).val(
                 $("#has_subscription_" + chart.chart_id).val() != "true");
@@ -587,16 +651,19 @@ $(document).ready(function () {
 
         if (this.chart_data.user) { // Is authenticated.
             if (this.chart_data.user.start_date) {
-                $("#start_date_" + this.chart_id).val(
-                    this.chart_data.user.start_date);
+                available_options = $.map($("#start_date_" + this.chart_id + " option"), function(e) { return e.value; });
+                if (available_options.indexOf(this.chart_data.user.start_date) > -1) {
+                    $("#start_date_" + this.chart_id).val(
+                        this.chart_data.user.start_date);
+                }
             }
             if (this.chart_data.user.is_legend_visible == false) {
                 $("#is_legend_visible_" + this.chart_id).prop("checked",
                                                               false);
             }
-            if (this.chart_data.user.toggle_percentage == true) {
-                $("#is_percentage_" + this.chart_id).prop("checked",
-                                                              true);
+            if (!this.chart_data.user.is_delta) {
+                $("#is_delta_" + this.chart_id).prop("checked",
+                                                     false);
             }
 
             this.set_subscription_link(this.chart_data.user.has_subscription);
@@ -626,8 +693,8 @@ $(document).ready(function () {
                 csrfmiddlewaretoken: csrf_token,
                 start_date: $("#start_date_" + this.chart_id).val(),
                 is_legend_visible: $("#is_legend_visible_" + this.chart_id).prop("checked"),
+                is_delta: $("#is_delta_" + this.chart_id).prop("checked"),
                 has_subscription: $("#has_subscription_" + this.chart_id).val(),
-                toggle_percentage: $("#is_percentage_" + this.chart_id).prop("checked"),
                 visible_chart_test_id: visible_chart_test_id,
                 visible_attribute_name: attr_name,
             },
@@ -667,10 +734,10 @@ $(document).ready(function () {
         // number/date boundaries.
         start_number = $("#start_date_" + this.chart_id).val();
         end_number = $("#end_date_" + this.chart_id).val();
-        if (this.chart_data.has_build_numbers) {
-            build_number = parseInt(build_number);
-            start_number = parseInt(start_number);
-            end_number = parseInt(end_number);
+        if (this.chart_data.is_keys_numeric) {
+            build_number = parseFloat(build_number);
+            start_number = parseFloat(start_number);
+            end_number = parseFloat(end_number);
         }
         if (build_number <= end_number && build_number >= start_number) {
 	    return true;
@@ -703,8 +770,30 @@ $(document).ready(function () {
 
         // Dates on the x-axis.
         dates = [];
-        // Store all build numbers
+
+
+        // Get all build numbers to be used as tick labels.
         build_numbers = [];
+        for (iter in this.chart_data.test_data) {
+
+	    row = this.chart_data.test_data[iter];
+
+            if (this.chart_data.xaxis_attribute) {
+                build_number = row["attribute"];
+            } else {
+	        build_number = row["number"].split('.')[0];
+            }
+
+            if (this.test_build_number(build_number)) {
+
+	        if (!isNumeric(build_number) && !this.chart_data.xaxis_attribute) {
+	            build_number = format_date(build_number);
+	        }
+                if (build_numbers.indexOf(build_number) == -1) {
+	            build_numbers.push(build_number);
+                }
+            }
+        }
 
         // Grid maximum and minimum values for y axis.
         var y_max = - Number.MAX_VALUE;
@@ -714,14 +803,26 @@ $(document).ready(function () {
         for (iter in this.chart_data.test_data) {
 
 	    row = this.chart_data.test_data[iter];
-            build_number = row["number"].split(".")[0];
 
-            // If some of the filters have build_number_attribute, ignore
+            if (this.chart_data.xaxis_attribute) {
+                if (row["attribute"]) {
+                    build_number = row["attribute"];
+                } else {
+                    continue;
+                }
+            } else {
+                build_number = row["number"].split(".")[0];
+            }
+
+            // If some of the filters have build_number attribute, ignore
             // others which don't.
-            if (this.chart_data.has_build_numbers && !isNumeric(build_number)) {
+            if (this.chart_data.is_keys_numeric && !isNumeric(build_number)) {
                 continue;
             }
 
+            // Fix json escaping.
+            row["link"] = row["link"].replace("\\\\\\", "");
+
             test_filter_id = row["test_filter_id"];
             if (this.test_build_number(build_number)) {
 
@@ -729,12 +830,8 @@ $(document).ready(function () {
                 iter = plot_data[test_filter_id]["data"].length;
 
                 if (this.chart_data["chart_type"] == "pass/fail") {
-                    if ($("#is_percentage_" + this.chart_id).prop("checked") == true) {
-                        value = parseFloat(row["passes"]/row["total"]).toFixed(4) * 100;
-                        // Happens when total is 0.
-                        if (isNaN(value)) {
-                            value = 0;
-                        }
+                    if (this.chart_data["is_percentage"] == true) {
+                        value = row["percentage"];
                         tooltip = "Pass rate: " + value + "%";
                     } else {
                         value = row["passes"];
@@ -743,7 +840,11 @@ $(document).ready(function () {
                     }
 
                 } else if (this.chart_data["chart_type"] == "measurement") {
-                    value = row["measurement"];
+                    value = parseFloat(row["measurement"]);
+                    if (isNaN(value)) {
+                        // Ignore plot point where measurement is non-numeric.
+                        continue;
+                    }
                     tooltip = "Value: " + value;
 
                 } else if (this.chart_data["chart_type"] == "attributes") {
@@ -751,7 +852,6 @@ $(document).ready(function () {
                     tooltip = "Value: " + value;
                 }
 
-
                 tooltip += "<br>";
                 label = "";
 
@@ -792,17 +892,26 @@ $(document).ready(function () {
                 plot_data[test_filter_id]["chart_test_id"] = row["chart_test_id"];
 
                 // Add the data.
-                if (this.chart_data.has_build_numbers) {
+                if (this.chart_data.is_keys_numeric) {
                     insert_data_item(build_number, [build_number, value],
                                      plot_data[test_filter_id]["data"]);
                     insert_data_item(build_number, label,
                                      plot_data[test_filter_id]["labels"]);
                     meta_key = build_number + "_" +  value;
                     plot_data[test_filter_id]["meta"][meta_key] = meta_item;
-                    build_numbers.push(build_number);
 
+                } else if (this.chart_data.xaxis_attribute) {
+                    key = build_numbers.indexOf(row["attribute"]);
+                    data_item = [key, value];
+                    insert_data_item(key, [key, value],
+                                     plot_data[test_filter_id]["data"]);
+                    insert_data_item(key, label,
+                                     plot_data[test_filter_id]["labels"]);
+                    // Meta keys are made unique by concatination.
+                    plot_data[test_filter_id]["meta"][data_item.join("_")] =
+                        meta_item;
                 } else {
-                    date = row["date"].split(".")[0].split(" ").join("T");
+                    date = row["number"].split(".")[0].split(" ").join("T");
                     key = Date.parse(date);
                     dates.push(key);
                     data_item = [key, value];
@@ -865,6 +974,48 @@ $(document).ready(function () {
         // Pack data in series for plot display.
         for (var i in sorted_filter_ids) {
             test_filter_id = sorted_filter_ids[i];
+
+            // Delta reporting, calculate diferences.
+            if ($("#is_delta_" + this.chart_id).prop("checked")) {
+                var new_data = [];
+                var new_meta_keys = [];
+                var tooltips = [];
+                for (j in plot_data[test_filter_id]["data"]) {
+                    if (j == 0) { //first element is auto-set to 0.
+                        new_value = 0;
+                        new_data.push(
+                            plot_data[test_filter_id]["data"][j].slice());
+                        new_data[0][1] = new_value;
+
+                    } else {
+                        new_value = +parseFloat(plot_data[test_filter_id]["data"][j][1] -
+                                     plot_data[test_filter_id]["data"][j-1][1]).toFixed(2);
+                        new_data.push(
+                            [plot_data[test_filter_id]["data"][j][0],
+                             new_value]);
+
+                    }
+                    // Set metadata key for first element.
+                    var meta_key = plot_data[test_filter_id]["data"][j][0]
+                        + "_" + plot_data[test_filter_id]["data"][j][1];
+                    new_meta_keys[meta_key] = new_data[j][0] + "_" +
+                        new_data[j][1];
+                    tooltips[meta_key] =
+                        plot_data[test_filter_id]["meta"][meta_key]["tooltip"] + "Delta: " + new_value + "<br>";
+                }
+                plot_data[test_filter_id]["data"] = new_data;
+
+                // Need to update metadata keys as well.
+                var new_metadata = [];
+                for (j in plot_data[test_filter_id]["meta"]) {
+                    plot_data[test_filter_id]["meta"][j]["tooltip"] =
+                        tooltips[j];
+                    new_metadata[new_meta_keys[j]] =
+                        plot_data[test_filter_id]["meta"][j];
+                }
+                plot_data[test_filter_id]["meta"] = new_metadata;
+            }
+
             if (this.legend_items.length != sorted_filter_ids.length) {
 
                 // Load hidden tests data.
@@ -932,28 +1083,9 @@ $(document).ready(function () {
             });
         }
 
-        // Get all build numbers to be used as tick labels.
-        build_numbers = [];
-        for (iter in this.chart_data.test_data) {
-
-	    row = this.chart_data.test_data[iter];
-
-	    build_number = row["number"].split('.')[0];
-
-            if (this.test_build_number(build_number)) {
-
-	        if (!isNumeric(build_number)) {
-	            build_number = format_date(build_number);
-	        }
-                if (build_numbers.indexOf(build_number) == -1) {
-	            build_numbers.push(build_number);
-                }
-            }
-        }
-
         // Add target goal dashed line to the plot.
         if (this.chart_data["target_goal"] != null) {
-            if ($("#is_percentage_" + this.chart_id).prop("checked") == true) {
+            if (this.chart_data["is_percentage"] == true) {
                 target_goal = parseFloat(this.chart_data["target_goal"]/y_max_pass).toFixed(4) * 100;
             } else {
                 target_goal = this.chart_data["target_goal"];
@@ -961,7 +1093,7 @@ $(document).ready(function () {
 
 	    goal_data = [];
 
-            if (this.chart_data.has_build_numbers) {
+            if (this.chart_data.is_keys_numeric || this.chart_data.xaxis_attribute) {
 	        for (var i in build_numbers) {
 	            goal_data.push([build_numbers[i], target_goal]);
 	        }
@@ -995,7 +1127,7 @@ $(document).ready(function () {
         }
 
         y_label = "";
-        if (this.chart_data["chart_type"] == "measurement") {
+        if (this.chart_data["chart_type"] == "pass/fail") {
             y_label = "Pass/Fail";
         } else if (this.chart_data["chart_type"] == "measurement") {
             if (this.chart_data.test_data[0]) {
@@ -1005,11 +1137,11 @@ $(document).ready(function () {
             }
         }
 
-        if (this.chart_data.has_build_numbers) {
+        if (this.chart_data.is_keys_numeric || this.chart_data.xaxis_attribute) {
 
             chart_data = this.chart_data;
             tick_formatter = function(val, axis) {
-                    if (chart_data.has_build_numbers) {
+                    if (chart_data.is_keys_numeric) {
                         return val;
                     } else {
                         return build_numbers[val];
@@ -1017,7 +1149,7 @@ $(document).ready(function () {
             }
 
             xaxis = {
-                tickDecimals: 0,
+                tickDecimals: (this.chart_data.is_keys_decimal)? 2:0,
                 tickFormatter: tick_formatter,
             };
 
@@ -1069,11 +1201,11 @@ $(document).ready(function () {
         // We cannot apply autoscaleMargin for y axis since y_max and y_min
         // are explicitely set. Therefore we will manually increase/decrease
         // the limits.
-        y_max *= 1.1;
-        y_min *= 0.9;
+        y_max += (0.1 * Math.abs(y_max));
+        y_min -= (0.1 * Math.abs(y_min));
 
-        if ($("#is_percentage_" + this.chart_id).prop("checked") == true) {
-            options["yaxis"]["max"] = 105;
+        if (this.chart_data["is_percentage"] == true) {
+            options["yaxis"]["max"] = 110;
             options["yaxis"]["min"] = 0;
         } else {
             options["yaxis"]["max"] = y_max;
@@ -1171,6 +1303,30 @@ $(document).ready(function () {
         return !isNaN(parseFloat(n)) && isFinite(n);
     }
 
+    is_array_numeric = function(arr) {
+        // Check if all items in array are numeric.
+        var is_array_numeric = true;
+        for (iter in arr) {
+            if (!isNumeric(arr[iter])) {
+                is_array_numeric = false;
+                break;
+            }
+        }
+        return is_array_numeric;
+    }
+
+    is_any_element_decimal = function(arr) {
+        // Check if any of the elements in array are decimal.
+        var is_decimal = false;
+        for (iter in arr) {
+            if (arr[iter].indexOf(".") != -1) {
+                is_decimal = true;
+                break;
+            }
+        }
+        return is_decimal;
+    }
+
     isValidUrl = function(url) {
         return url.match(/^https?:\/\/[a-z0-9-\.]+\.[a-z]{2,4}\/?([^\s<>\#%"\,\{\}\\|\\\^\[\]`]+)?$/);
     }
@@ -1178,13 +1334,13 @@ $(document).ready(function () {
     insert_data_item = function(key, value, data) {
         // Insert item at the sorted position in the data.
         // data represents list of two-value lists.
-        if (data.length == 0 || parseInt(key) <= parseInt(data[0][0])) {
+        if (data.length == 0 || parseFloat(key) <= parseFloat(data[0][0])) {
             data.splice(0, 0, value);
             return;
         }
         for (var i=0; i < data.length-1; i++) {
-            if (parseInt(key) > parseInt(data[i][0]) &&
-                parseInt(key) <= parseInt(data[i+1][0])) {
+            if (parseFloat(key) > parseFloat(data[i][0]) &&
+                parseFloat(key) <= parseFloat(data[i+1][0])) {
                 data.splice(i+1, 0, value);
                 return;
             }
@@ -1397,6 +1553,7 @@ $(document).ready(function () {
                     link_bug_url = testresult_link_bug_url;
                     unlink_bug_url = testresult_unlink_bug_url;
                 } else {
+                    add_bug_dialog.find('input[name=relative_index]').val("");
                     link_bug_url = testrun_link_bug_url;
                     unlink_bug_url = testrun_unlink_bug_url;
                 }
