@@ -2,19 +2,19 @@
 #
 # Author: Zygmunt Krynicki <zygmunt.krynicki@linaro.org>
 #
-# This file is part of Launch Control.
+# This file is part of Lava Dashboard.
 #
-# Launch Control is free software: you can redistribute it and/or modify
+# Lava Dashboard is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License version 3
 # as published by the Free Software Foundation
 #
-# Launch Control is distributed in the hope that it will be useful,
+# Lava Dashboard is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with Launch Control.  If not, see <http://www.gnu.org/licenses/>.
+# along with Lava Dashboard.  If not, see <http://www.gnu.org/licenses/>.
 
 """
 Views for the Dashboard application
@@ -29,6 +29,7 @@ import tempfile
 
 from django.contrib.auth.decorators import login_required
 from django.contrib.auth.models import User
+from django.contrib.contenttypes.models import ContentType
 from django.contrib.sites.models import Site
 from django.core import serializers
 from django.core.exceptions import PermissionDenied
@@ -56,16 +57,21 @@ from dashboard_app.models import (
     Attachment,
     Bundle,
     BundleStream,
+    ImageReport,
+    ImageReportChart,
+    ImageChartUser,
+    NamedAttribute,
     Tag,
     Test,
     TestCase,
     TestResult,
     TestRun,
+    TestRunFilter,
     TestDefinition,
     BugLink,
 )
 from lava_scheduler_app.models import (
-    TestJob
+    TestJob,
 )
 from dashboard_app.views.tables import (
     BundleStreamTable,
@@ -75,6 +81,9 @@ from dashboard_app.views.tables import (
     TestTable,
     TestDefinitionTable,
 )
+from dashboard_app.views.filters.tables import PublicFiltersTable
+from dashboard_app.views.image_reports.tables import UserImageReportTable
+
 from django_tables2 import (
     Attrs,
     Column,
@@ -153,6 +162,31 @@ class MyBundleStreamView(BundleStreamView):
         return BundleStream.objects.owned_by_principal(self.request.user).order_by('pathname')
 
 
+class SubscribedFiltersView(LavaView):
+
+    def __init__(self, request, **kwargs):
+        super(SubscribedFiltersView, self).__init__(request, **kwargs)
+
+    def stream_query(self, term):
+        streams = BundleStream.objects.filter(pathname__contains=term)
+        return Q(bundle_streams__in=streams)
+
+    def get_queryset(self):
+        return TestRunFilter.objects.filter(
+            public=True,
+            testrunfiltersubscription__user=self.request.user)
+
+
+class SubscribedImageReportView(LavaView):
+
+    def get_queryset(self):
+
+        return ImageReport.objects.filter(
+            imagereportchart__imagechartuser__user=self.request.user,
+            imagereportchart__imagechartuser__has_subscription=True
+        )
+
+
 @BreadCrumb("Dashboard", parent=lava_index)
 def index(request):
     return render_to_response(
@@ -313,7 +347,7 @@ def bundle_list_export(request, pathname):
     # Add results columns from denormalization object.
     bundle_keys.extend(["pass", "fail", "total"])
 
-    with open(file_path, 'w+') as csv_file:
+    with open(file_path, 'wb+') as csv_file:
         out = csv.DictWriter(csv_file, quoting=csv.QUOTE_ALL,
                              extrasaction='ignore',
                              fieldnames=bundle_keys)
@@ -439,7 +473,7 @@ def bundle_export(request, pathname, content_sha1):
     # Add bug link
     test_run_keys.append("bug_link")
 
-    with open(file_path, 'w+') as csv_file:
+    with open(file_path, 'wb+') as csv_file:
         out = csv.DictWriter(csv_file, quoting=csv.QUOTE_ALL,
                              extrasaction='ignore',
                              fieldnames=test_run_keys)
@@ -450,7 +484,7 @@ def bundle_export(request, pathname, content_sha1):
             test_run_denorm = test_run.denormalization
             test_run_dict = test_run.__dict__.copy()
             test_run_dict.update(test_run_denorm.__dict__)
-            test_run_dict["test"] = test_run.test.test_id
+            test_run_dict["test"] = test_run.test.test_id.encode('utf-8')
             test_run_dict["test_params"] = test_run.get_test_params()
             test_run_dict["device"] = test_run.show_device()
             test_run_dict["bug_link"] = " ".join([b.bug_link for b in test_run.bug_links.all()])
@@ -647,7 +681,7 @@ def test_run_export(request, pathname, content_sha1, analyzer_assigned_uuid):
 
     test_result_keys.append('bug_link')
 
-    with open(file_path, 'w+') as csv_file:
+    with open(file_path, 'wb+') as csv_file:
         out = csv.DictWriter(csv_file, quoting=csv.QUOTE_ALL,
                              extrasaction='ignore',
                              fieldnames=test_result_keys)
@@ -715,6 +749,62 @@ def test_run_hardware_context(request, pathname, content_sha1, analyzer_assigned
         }, RequestContext(request))
 
 
+@login_required
+def test_run_update_attribute(request, pathname, content_sha1,
+                              analyzer_assigned_uuid):
+
+    if request.method != 'POST':
+        raise PermissionDenied
+
+    test_run = get_restricted_object(
+        TestRun,
+        lambda test_run: test_run.bundle.bundle_stream,
+        request.user,
+        analyzer_assigned_uuid=analyzer_assigned_uuid
+    )
+
+    content_type_id = ContentType.objects.get_for_model(TestRun).id
+
+    if not request.POST.get("pk"):
+        named_attribute = NamedAttribute()
+        named_attribute.object_id = test_run.id
+        named_attribute.content_type_id = content_type_id
+        named_attribute.is_manual = True
+
+    else:
+        try:
+            named_attribute = NamedAttribute.objects.get(
+                pk=request.POST.get("pk"))
+        except NamedAttribute.DoesNotExist:
+            named_attribute = NamedAttribute()
+            named_attribute.object_id = test_run.id
+            named_attribute.content_type_id = content_type_id
+            named_attribute.is_manual = True
+
+    named_attribute.name = request.POST.get('name')
+    named_attribute.value = request.POST.get('value')
+    named_attribute.save()
+    data = serializers.serialize('json', [named_attribute])
+    return HttpResponse(data, content_type='application/json')
+
+
+@login_required
+def test_run_remove_attribute(request, pathname, content_sha1,
+                              analyzer_assigned_uuid):
+
+    if request.method != 'POST':
+        raise PermissionDenied
+
+    named_attribute = NamedAttribute.objects.get(
+        pk=request.POST.get('pk'))
+    if not named_attribute.is_manual:
+        return HttpResponse("error", status=403)
+    else:
+        named_attribute.delete()
+        data = request.POST.get('pk')
+        return HttpResponse(data, content_type='application/json')
+
+
 @BreadCrumb(
     "Details of result {relative_index}",
     parent=test_run_detail,
@@ -766,6 +856,67 @@ def test_result_update_comments(request, pathname, content_sha1,
     return HttpResponse(data, content_type='application/json')
 
 
+@login_required
+def test_result_update_attribute(request, pathname, content_sha1,
+                                 analyzer_assigned_uuid, relative_index):
+
+    if request.method != 'POST':
+        raise PermissionDenied
+
+    test_run = get_restricted_object(
+        TestRun,
+        lambda test_run: test_run.bundle.bundle_stream,
+        request.user,
+        analyzer_assigned_uuid=analyzer_assigned_uuid
+    )
+    try:
+        test_result = test_run.test_results.select_related('fig').get(
+            relative_index=relative_index)
+    except TestResult.DoesNotExist:
+        raise Http404
+
+    content_type_id = ContentType.objects.get_for_model(TestResult).id
+
+    if not request.POST.get("pk"):
+        named_attribute = NamedAttribute()
+        named_attribute.object_id = test_result.id
+        named_attribute.content_type_id = content_type_id
+        named_attribute.is_manual = True
+
+    else:
+        try:
+            named_attribute = NamedAttribute.objects.get(
+                pk=request.POST.get("pk"))
+        except NamedAttribute.DoesNotExist:
+            named_attribute = NamedAttribute()
+            named_attribute.object_id = test_result.id
+            named_attribute.content_type_id = content_type_id
+            named_attribute.is_manual = True
+
+    named_attribute.name = request.POST.get('name')
+    named_attribute.value = request.POST.get('value')
+    named_attribute.save()
+    data = serializers.serialize('json', [named_attribute])
+    return HttpResponse(data, content_type='application/json')
+
+
+@login_required
+def test_result_remove_attribute(request, pathname, content_sha1,
+                                 analyzer_assigned_uuid, relative_index):
+
+    if request.method != 'POST':
+        raise PermissionDenied
+
+    named_attribute = NamedAttribute.objects.get(
+        pk=request.POST.get('pk'))
+    if not named_attribute.is_manual:
+        return HttpResponse("error", status=403)
+    else:
+        named_attribute.delete()
+        data = request.POST.get('pk')
+        return HttpResponse(data, content_type='application/json')
+
+
 def attachment_download(request, pk):
     attachment = get_restricted_object(
         Attachment,
@@ -804,6 +955,52 @@ def attachment_view(request, pk):
         }, RequestContext(request))
 
 
+@BreadCrumb("Subscriptions", parent=index)
+def my_subscriptions(request):
+
+    prefix = "filter_"
+    filter_view = SubscribedFiltersView(request, model=TestRunFilter,
+                                        table_class=PublicFiltersTable)
+    filters_table = PublicFiltersTable(
+        request.user,
+        filter_view.get_table_data(prefix),
+        prefix=prefix
+    )
+    config = RequestConfig(request)
+    config.configure(filters_table)
+
+    search_data = filters_table.prepare_search_data(filter_view)
+    discrete_data = filters_table.prepare_discrete_data(filter_view)
+    terms_data = filters_table.prepare_terms_data(filter_view)
+    times_data = filters_table.prepare_times_data(filter_view)
+
+    prefix = "report_"
+    report_view = SubscribedImageReportView(request, model=ImageReportChart,
+                                            table_class=UserImageReportTable)
+    report_table = UserImageReportTable(report_view.get_table_data(prefix),
+                                        prefix=prefix)
+    config = RequestConfig(request)
+    config.configure(report_table)
+
+    search_data.update(report_table.prepare_search_data(report_view))
+    discrete_data.update(report_table.prepare_discrete_data(report_view))
+    terms_data.update(report_table.prepare_terms_data(report_view))
+    times_data.update(report_table.prepare_times_data(report_view))
+
+    return render_to_response(
+        'dashboard_app/subscribed_list.html', {
+            'filters_table': filters_table,
+            'report_table': report_table,
+            "terms_data": terms_data,
+            "search_data": search_data,
+            "times_data": times_data,
+            "discrete_data": discrete_data,
+            'bread_crumb_trail': BreadCrumbTrail.leading_to(
+                my_subscriptions),
+        }, RequestContext(request)
+    )
+
+
 def redirect_to(request, object, trailing):
     url = object.get_absolute_url() + trailing
     qs = request.META.get('QUERY_STRING')
