@@ -2,19 +2,19 @@
 #
 # Author: Michael Hudson-Doyle <michael.hudson@linaro.org>
 #
-# This file is part of Launch Control.
+# This file is part of Lava Dashboard.
 #
-# Launch Control is free software: you can redistribute it and/or modify
+# Lava Dashboard is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License version 3
 # as published by the Free Software Foundation
 #
-# Launch Control is distributed in the hope that it will be useful,
+# Lava Dashboard is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with Launch Control.  If not, see <http://www.gnu.org/licenses/>.
+# along with Lava Dashboard.  If not, see <http://www.gnu.org/licenses/>.
 
 import datetime
 import operator
@@ -36,6 +36,10 @@ from dashboard_app.models import (
 
 class UserFiltersTable(LavaTable):
 
+    def __init__(self, user, *args, **kwargs):
+        super(UserFiltersTable, self).__init__(*args, **kwargs)
+        self.user = user
+
     name = tables.TemplateColumn('''
     <a href="{{ record.get_absolute_url }}">{{ record.name }}</a>
     ''')
@@ -81,15 +85,15 @@ class UserFiltersTable(LavaTable):
     ''')
     test.orderable = False
 
-    subscription = tables.Column()
+    subscription = tables.TemplateColumn('''{{}}''')
     subscription.orderable = False
 
     def render_subscription(self, record):
         try:
             sub = TestRunFilterSubscription.objects.get(
                 user=self.user, filter=record)
-        except TestRunFilterSubscription.DoesNotExist:
-            return "None"
+        except (TypeError, TestRunFilterSubscription.DoesNotExist):
+            return None
         else:
             return sub.get_level_display()
 
@@ -114,8 +118,8 @@ class PublicFiltersTable(UserFiltersTable):
     <a href="{{ record.get_absolute_url }}">~{{ record.owner.username }}/{{ record.name }}</a>
     ''')
 
-    def __init__(self, *args, **kw):
-        super(PublicFiltersTable, self).__init__(*args, **kw)
+    def __init__(self, user, *args, **kw):
+        super(PublicFiltersTable, self).__init__(user, *args, **kw)
 
     class Meta(LavaTable.Meta):
         exclude = (
@@ -169,7 +173,7 @@ class SpecificCaseColumn(tables.Column):
         for result in results:
             if result.test_case_id != self.test_case.id:
                 continue
-            if result.result == result.RESULT_PASS and result.units:
+            if result.result == result.RESULT_PASS and result.measurement:
                 s = '%s %s' % (result.measurement, result.units)
             else:
                 s = result.RESULT_MAP[result.result]
