@@ -2,19 +2,19 @@
 #
 # Author: Stevan Radakovic <stevan.radakovic@linaro.org>
 #
-# This file is part of Launch Control.
+# This file is part of Lava Dashboard.
 #
-# Launch Control is free software: you can redistribute it and/or modify
+# Lava Dashboard is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License version 3
 # as published by the Free Software Foundation
 #
-# Launch Control is distributed in the hope that it will be useful,
+# Lava Dashboard is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with Launch Control.  If not, see <http://www.gnu.org/licenses/>.
+# along with Lava Dashboard.  If not, see <http://www.gnu.org/licenses/>.
 
 from django import forms
 
@@ -49,7 +49,16 @@ class ImageReportChartForm(forms.ModelForm):
     class Meta:
         model = ImageReportChart
         fields = '__all__'
-        widgets = {'image_report': forms.HiddenInput}
+        widgets = {
+            'image_report': forms.HiddenInput,
+            'relative_index': forms.HiddenInput
+        }
+
+    xaxis_attribute_changed = forms.BooleanField(
+        widget=forms.widgets.HiddenInput,
+        required=False,
+        initial=False
+    )
 
     def __init__(self, user, *args, **kwargs):
         super(ImageReportChartForm, self).__init__(*args, **kwargs)
@@ -57,6 +66,23 @@ class ImageReportChartForm(forms.ModelForm):
             self.fields['chart_type'].label = ""
             self.fields['chart_type'].widget = forms.HiddenInput()
 
+        if not self.instance.imagechartfilter_set.all() or \
+           self.instance.chart_type == "attributes":
+            self.fields['xaxis_attribute'].label = ""
+            self.fields['xaxis_attribute'].widget = forms.HiddenInput()
+        else:
+            custom_attrs = self.instance.get_supported_attributes(user)
+
+            if custom_attrs:
+                self.fields['xaxis_attribute'] = forms.TypedChoiceField(
+                    required=False,
+                    choices=[("", "----")] +
+                    [(attr, attr) for attr in custom_attrs],
+                )
+            else:
+                self.fields['xaxis_attribute'].label = ""
+                self.fields['xaxis_attribute'].widget = forms.HiddenInput()
+
     def save(self, commit=True, **kwargs):
         instance = super(ImageReportChartForm,
                          self).save(commit=commit, **kwargs)
