@@ -16,12 +16,12 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with LAVA Server.  If not, see <http://www.gnu.org/licenses/>.
 
-from lava_server.extension import LavaServerExtension
+from lava_server.extension import DeprecatedExtension
 
 
-class ProjectExtension(LavaServerExtension):
+class ProjectExtension(DeprecatedExtension):
     """
-    Extension adding project support
+    This has been deprecated but is just in place for south database migrations
     """
 
     @property
@@ -31,18 +31,3 @@ class ProjectExtension(LavaServerExtension):
     @property
     def name(self):
         return "Projects"
-
-    @property
-    def main_view_name(self):
-        return "lava.project.root"
-
-    @property
-    def description(self):
-        return "Project support for LAVA"
-
-    @property
-    def version(self):
-        import lava_projects
-        import versiontools
-        return versiontools.format_version(
-            lava_projects.__version__, hint=lava_projects)
