@@ -46,8 +46,10 @@ class RestrictedIDLinkColumn(IDLinkColumn):
             device_type = record.actual_device.device_type
         elif record.requested_device:
             device_type = record.requested_device.device_type
-        else:
+        elif record.requested_device_type:
             device_type = record.requested_device_type
+        else:
+            return pklink(record)
 
         if len(device_type.devices_visible_to(table.context.get('request').user)) == 0:
             return "Unavailable"
@@ -239,6 +241,8 @@ class IndexJobTable(JobTable):
 
     id = RestrictedIDLinkColumn(verbose_name="ID", accessor="id")
     device = tables.Column(accessor='device_sort')
+    submit_time = DateColumn()
+    end_time = DateColumn()
 
     def __init__(self, *args, **kwargs):
         super(IndexJobTable, self).__init__(*args, **kwargs)
@@ -276,6 +280,8 @@ class FailedJobTable(JobTable):
     duration.orderable = False
     failure_tags = TagsColumn()
     failure_comment = tables.Column()
+    submit_time = DateColumn()
+    end_time = DateColumn()
 
     def __init__(self, *args, **kwargs):
         super(FailedJobTable, self).__init__(*args, **kwargs)
@@ -297,6 +303,8 @@ class OverviewJobsTable(JobTable):
     device = tables.Column(accessor='device_sort')
     duration = tables.Column(accessor='duration_sort')
     duration.orderable = False
+    submit_time = DateColumn()
+    end_time = DateColumn()
 
     def __init__(self, *args, **kwargs):
         super(OverviewJobsTable, self).__init__(*args, **kwargs)
@@ -317,13 +325,18 @@ class RecentJobsTable(JobTable):
     log_level = tables.Column(accessor="definition", verbose_name="Log level")
     duration = tables.Column(accessor='duration_sort')
     duration.orderable = False
+    submit_time = DateColumn()
+    end_time = DateColumn()
 
     def __init__(self, *args, **kwargs):
         super(RecentJobsTable, self).__init__(*args, **kwargs)
         self.length = 10
 
     def render_log_level(self, record):
-        data = json.loads(record.definition)
+        try:
+            data = json.loads(record.definition)
+        except ValueError:
+            return "debug"
         try:
             data['logging_level']
         except KeyError:
@@ -522,6 +535,8 @@ class WorkerTable(tables.Table):
         self.length = 10
         self.show_help = True
 
+    last_master_scheduler_tick = DateColumn()
+
     hostname = tables.TemplateColumn('''
     {% if record.too_long_since_last_heartbeat %}
     <span class="glyphicon glyphicon-thumbs-down text-danger"></span>
@@ -635,6 +650,8 @@ class QueueJobsTable(JobTable):
     for {{ record.submit_time|timesince }}
     ''')
     in_queue.orderable = False
+    submit_time = DateColumn()
+    end_time = DateColumn()
 
     def __init__(self, *args, **kwargs):
         super(QueueJobsTable, self).__init__(*args, **kwargs)
