@@ -507,6 +507,117 @@ class TestTestJob(TestCaseWithFactory):
         self.assertEqual(definition_data['timeout'], job_data['timeout'])
         self.assertEqual(definition_data['health_check'], job_data['health_check'])
 
+    def test_from_json_and_user_repeat_parameter_expansion(self):
+        device_type = self.factory.make_device_type('base')
+        device = self.factory.make_device(device_type=device_type, hostname="generic")
+        repeat = 5
+        job_data = {
+            'timeout': 1,
+            'target': device.hostname,
+            'actions': [
+                {
+                    'command': 'lava_test_shell',
+                    'parameters': {
+                        'repeat': repeat,
+                        'testdef_repos': [
+                            {
+                                'git-repo': 'git://server/test.git',
+                                'testdef': 'testdef.yaml'
+                            }
+                        ],
+                    }
+                }
+            ],
+        }
+        job_json = simplejson.dumps(job_data, sort_keys=True, indent=4 * ' ')
+        job = TestJob.from_json_and_user(job_json, self.factory.make_user())
+        definition_data = simplejson.loads(job.definition)
+        self.assertEqual(len(definition_data['actions']), repeat)
+        self.assertEqual(job.status, TestJob.SUBMITTED)
+
+    def test_from_json_and_user_repeat_parameter_replace_with_repeat_count(self):
+        device_type = self.factory.make_device_type('base')
+        device = self.factory.make_device(device_type=device_type, hostname="generic")
+        repeat = 5
+        job_data = {
+            'timeout': 1,
+            'target': device.hostname,
+            'actions': [
+                {
+                    'command': 'lava_test_shell',
+                    'parameters': {
+                        'repeat': repeat,
+                        'testdef_repos': [
+                            {
+                                'git-repo': 'git://server/test.git',
+                                'testdef': 'testdef.yaml'
+                            }
+                        ],
+                    }
+                }
+            ],
+        }
+        job_json = simplejson.dumps(job_data, sort_keys=True, indent=4 * ' ')
+        job = TestJob.from_json_and_user(job_json, self.factory.make_user())
+        definition_data = simplejson.loads(job.definition)
+        self.assertEqual(len(definition_data['actions']), repeat)
+        for i in range(repeat):
+            self.assertEqual(definition_data['actions'][i]['parameters']['repeat_count'], i)
+            self.assertNotIn('repeat', definition_data['actions'][i]['parameters'])
+        self.assertEqual(job.status, TestJob.SUBMITTED)
+
+    def test_from_json_and_user_repeat_parameter_zero(self):
+        device_type = self.factory.make_device_type('base')
+        device = self.factory.make_device(device_type=device_type, hostname="generic")
+        repeat = 0
+        job_data = {
+            'timeout': 1,
+            'target': device.hostname,
+            'actions': [
+                {
+                    'command': 'lava_test_shell',
+                    'parameters': {
+                        'repeat': repeat,
+                        'testdef_repos': [
+                            {
+                                'git-repo': 'git://server/test.git',
+                                'testdef': 'testdef.yaml'
+                            }
+                        ],
+                    }
+                }
+            ],
+        }
+        job_json = simplejson.dumps(job_data, sort_keys=True, indent=4 * ' ')
+        job = TestJob.from_json_and_user(job_json, self.factory.make_user())
+        definition_data = simplejson.loads(job.definition)
+        self.assertEqual(len(definition_data['actions']), 1)
+        self.assertNotIn('repeat_count', definition_data['actions'][0]['parameters'])
+        self.assertNotIn('repeat', definition_data['actions'][0]['parameters'])
+        self.assertEqual(job.status, TestJob.SUBMITTED)
+
+    def test_from_json_and_user_repeat_parameter_not_supported(self):
+        device_type = self.factory.make_device_type('base')
+        device = self.factory.make_device(device_type=device_type, hostname="generic")
+        repeat = 1
+        job_data = {
+            'timeout': 1,
+            'target': device.hostname,
+            'actions': [
+                {
+                    'command': 'deploy_linaro_image',
+                    'parameters': {
+                        'repeat': repeat,
+                        'image': 'file:///pathto/image.img.gz'
+                    }
+                }
+            ],
+        }
+        job_json = simplejson.dumps(job_data, sort_keys=True, indent=4 * ' ')
+        self.assertRaises(
+            ValueError, TestJob.from_json_and_user, job_json,
+            self.factory.make_user())
+
 
 class TestHiddenTestJob(TestCaseWithFactory):
 
@@ -637,6 +748,76 @@ class TestSchedulerAPI(TestCaseWithFactory):
         job = TestJob.objects.get(pk=job.pk)
         self.assertEqual(TestJob.CANCELED, job.status)
 
+    def test_json_vs_yaml(self):
+        """
+        Test that invalid JSON gets rejected but valid YAML is accepted as pipeline
+        """
+        user = User.objects.create_user('test', 'e@mail.invalid', 'test')
+        user.user_permissions.add(
+            Permission.objects.get(codename='add_testjob'))
+        user.save()
+        job = self.factory.make_testjob(submitter=user)
+        self.assertFalse(job.is_pipeline)
+        # "break" the JSON by dropping the closing } as JSON needs the complete file to validate
+        invalid_def = job.definition[:-2]
+        self.assertRaises(ValueError, json.loads, invalid_def)
+        server = self.server_proxy('test', 'test')
+        self.assertRaises(xmlrpclib.Fault, server.scheduler.submit_job, invalid_def)
+
+        invalid_yaml_def = """
+# Sample JOB definition for a KVM
+device_type: qemu
+job_name: kvm-pipeline
+priority: medium
+"""
+        self.assertRaises(ValueError, json.loads, invalid_yaml_def)
+        self.assertRaises(xmlrpclib.Fault, server.scheduler.submit_job, invalid_yaml_def)
+
+        yaml_def = """
+# Sample JOB definition for a KVM
+device_type: qemu
+job_name: kvm-pipeline
+priority: medium
+
+actions:
+
+    - deploy:
+        to: tmpfs
+        image: http://images.validation.linaro.org/kvm-debian-wheezy.img.gz
+        compression: gz
+        os: debian
+
+    - boot:
+        method: qemu
+        media: tmpfs
+        failure_retry: 2
+
+    - test:
+        name: kvm-basic-singlenode
+        definitions:
+            - repository: git://git.linaro.org/qa/test-definitions.git
+              from: git
+              path: ubuntu/smoke-tests-basic.yaml
+              # name: if not present, use the name from the YAML. The name can
+              # also be overriden from the actual commands being run by
+              # calling the lava-test-suite-name API call (e.g.
+              # `lava-test-suite-name FOO`).
+              name: smoke-tests
+"""
+        self.assertRaises(xmlrpclib.Fault, server.scheduler.submit_job, yaml_def)
+
+        device_type = self.factory.make_device_type('qemu')
+        device = self.factory.make_device(device_type=device_type, hostname="qemu1")
+        device.save()
+        self.assertFalse(device.is_pipeline)
+        self.assertRaises(xmlrpclib.Fault, server.scheduler.submit_job, yaml_def)
+        device = self.factory.make_device(device_type=device_type, hostname="qemu2", is_pipeline=True)
+        device.save()
+        self.assertTrue(device.is_pipeline)
+        job_id = server.scheduler.submit_job(yaml_def)
+        job = TestJob.objects.get(id=job_id)
+        self.assertTrue(job.is_pipeline)
+
 
 class TransactionTestCaseWithFactory(TransactionTestCase):
 
