@@ -78,6 +78,12 @@ urlpatterns = patterns(
     url(r'^job/(?P<pk>[0-9]+)/vmgroup_definition/plain$',
         'vmgroup_job_definition_plain',
         name='lava.scheduler.job.vmgroup_definition.plain'),
+    url(r'^job/(?P<pk>[0-9]+)/expanded_definition$',
+        'expanded_job_definition',
+        name='lava.scheduler.job.expanded_definition'),
+    url(r'^job/(?P<pk>[0-9]+)/expanded_definition/plain$',
+        'vmgroup_job_definition_plain',
+        name='lava.scheduler.job.expanded_definition.plain'),
     url(r'^job/(?P<pk>[0-9]+)/log_file$',
         'job_log_file',
         name='lava.scheduler.job.log_file'),
