@@ -20,13 +20,13 @@
 import os
 import re
 import copy
-import uuid
 import socket
 import urlparse
 import simplejson
 import models
 import subprocess
 import datetime
+import netifaces
 
 from collections import OrderedDict
 
@@ -37,6 +37,19 @@ from lava_server.settings.getsettings import Settings
 from lava_server.settings.config_file import ConfigFile
 
 
+def get_fqdn():
+    """Returns the fully qualified domain name.
+    """
+    host = socket.getfqdn()
+    try:
+        if bool(re.match("[-_a-zA-Z0-9.]+$", host)):
+            return host
+        else:
+            raise ValueError("Your FQDN contains invalid characters")
+    except ValueError as exc:
+        raise exc
+
+
 def rewrite_hostname(result_url):
     """If URL has hostname value as localhost/127.0.0.*, change it to the
     actual server FQDN.
@@ -45,7 +58,7 @@ def rewrite_hostname(result_url):
 
     See https://cards.linaro.org/browse/LAVA-611
     """
-    domain = socket.getfqdn()
+    domain = get_fqdn()
     try:
         site = Site.objects.get_current()
     except (Site.DoesNotExist, ImproperlyConfigured):
@@ -269,16 +282,22 @@ def get_lshw_out():
     return simplejson.dumps(lshw_out)
 
 
-def get_fqdn():
-    """Returns the fully qualified domain name.
-    """
-    return socket.getfqdn()
-
-
 def get_ip_address():
-    """Returns the IP address.
+    """Returns the IP address of the default interface, if found.
     """
-    return socket.gethostbyname_ex(socket.getfqdn())[2][0]
+    ip = '0.0.0.0'
+    gateways = netifaces.gateways()
+    if gateways:
+        default_gateway = gateways.get('default')
+        if default_gateway:
+            default_interface = default_gateway.get(netifaces.AF_INET)[1]
+            if default_interface:
+                default_interface_values = netifaces.ifaddresses(
+                    default_interface)
+                if default_interface_values:
+                    ip = default_interface_values.get(
+                        netifaces.AF_INET)[0].get('addr')
+    return ip
 
 
 def format_sw_info_to_html(data_dict):
@@ -426,3 +445,201 @@ def last_scheduler_tick():
     """Returns datetime.dateime object of last scheduler tick timestamp.
     """
     return __last_scheduler_tick
+
+
+def process_repeat_parameter(json_jobdata):
+    new_json = {}
+    new_actions = []
+    allowed_actions = ["delpoy_linaro_image", "deploy_image",
+                       "boot_linaro_image", "boot_linaro_android_image",
+                       "lava_test_shell", "lava_android_test_run",
+                       "lava_android_test_run_custom", "lava_android_test_run_monkeyrunner"]
+
+    # Take each action and expand it if repeat parameter is specified.
+    all_actions = json_jobdata["actions"]
+    for action in all_actions:
+        new_action = copy.deepcopy(action)
+        if 'parameters' in new_action \
+           and 'repeat' in new_action["parameters"]:
+            if new_action["command"] not in allowed_actions:
+                raise ValueError("Action '%s' can't be repeated" % new_action["command"])
+            repeat = new_action["parameters"]["repeat"]
+            new_action["parameters"].pop('repeat', None)
+            if repeat > 1:
+                for i in range(repeat):
+                    new_action["parameters"]["repeat_count"] = i
+                    new_actions.append(copy.deepcopy(new_action))
+            else:
+                new_actions.append(copy.deepcopy(new_action))
+        else:
+            new_actions.append(new_action)
+
+    new_json["timeout"] = json_jobdata["timeout"]
+    if json_jobdata.get("device_type", False):
+        new_json["device_type"] = json_jobdata["device_type"]
+    if json_jobdata.get("target", False):
+        new_json["target"] = json_jobdata["target"]
+    if json_jobdata.get("job_name", False):
+        new_json["job_name"] = json_jobdata["job_name"]
+    if json_jobdata.get("logging_level", False):
+        new_json["logging_level"] = json_jobdata["logging_level"]
+    if json_jobdata.get("priority", False):
+        new_json["priority"] = json_jobdata["priority"]
+    if json_jobdata.get("tags", False):
+        new_json["tags"] = json_jobdata["tags"]
+    if "health_check" in json_jobdata:
+        new_json["health_check"] = json_jobdata.get("health_check")
+    if "device_group" in json_jobdata:
+        new_json["device_group"] = json_jobdata.get("device_group")
+    if "vm_group" in json_jobdata:
+        new_json["vm_group"] = json_jobdata.get("vm_group")
+    new_json["actions"] = new_actions
+
+    return new_json
+
+
+def is_member(user, group):
+    return user.groups.filter(name='%s' % group).exists()
+
+
+def devicedictionary_to_jinja2(data_dict, extends):
+    """
+    Formats a DeviceDictionary as a jinja2 string dictionary
+    Arguments:
+        data_dict: the DeviceDictionary.to_dict()
+        extends: the name of the jinja2 device_type template file to extend.
+        (including file name extension / suffix) which jinja2 will later
+        assume to be in the jinja2 device_types folder
+    """
+    if type(data_dict) is not dict:
+        return None
+    data = u'{%% extends \'%s\' %%}\n' % extends
+    for key, value in data_dict.items():
+        if key == 'extends':
+            continue
+        data += u'{%% set %s = \'%s\' %%}\n' % (key, value)
+    return data
+
+
+def jinja2_to_devicedictionary(data_dict):
+    """
+    Do some string mangling to convert the template to a key value store
+    The reverse of lava_scheduler_app.utils.devicedictionary_to_jinja2
+    """
+    if type(data_dict) is not str:
+        return None
+    data = {}
+    for line in data_dict.replace('{% ', '').replace(' %}', '').split('\n'):
+        if line == '':
+            continue
+        if line.startswith('extends'):
+            base = line.replace('extends ', '')
+            base = base.replace('"', "'").replace("'", '')
+            data['extends'] = base
+        if line.startswith('set '):
+            key = line.replace('set ', '')
+            key = re.sub(' = .*$', '', key)
+            value = re.sub('^.* = ', '', line)
+            value = value.replace('"', "'").replace("'", '')
+            data[key] = value
+    return data
+
+
+def jinja_template_path(system=True):
+    """
+    Use the source code for jinja2 templates, e.g. for unit tests
+    """
+    path = '/etc/lava-server/dispatcher-config/'
+    if os.path.exists(path) and system:
+        return path
+    path = os.path.realpath(os.path.join(os.path.dirname(__file__), '../etc/dispatcher-config/'))
+    if not os.path.exists(path):
+        raise RuntimeError("Misconfiguration of jinja templates")
+    return path
+
+
+def split_multinode_yaml(submission, target_group):  # pylint: disable=too-many-branches,too-many-locals
+    """
+    Handles the lava-multinode protocol requirements.
+    Uses the multinode protocol requirements to generate as many YAML
+    snippets as are required to create TestJobs for the multinode submission.
+    Each multinode YAML submission is only split once for all roles and all sub jobs.
+    parameters:
+      submission - the dictionary of the submission YAML.
+      device_dict - the dictionary mapping device hostnames to roles
+      target_group - the uuid of the multinode group
+    return:
+      None on error or a dictionary of job roles.
+      key: role
+      value: list of jobs to be created for that role.
+     """
+    # the list of devices cannot be definite here, only after devices have been reserved
+
+    copies = [
+        'context',
+        'job_name',
+        'timeouts',
+        'priority',
+    ]
+    skip = ['role', 'roles']
+    maps = ['device_type', 'count']
+
+    roles = {}
+    actions = {}
+    subid = 0
+
+    # FIXME: check structure using a schema
+
+    group_size = len(submission['protocols']['lava-multinode']['roles'].keys())
+
+    # populate the lava-multinode protocol metadata
+    for role, value in submission['protocols']['lava-multinode']['roles'].iteritems():
+        roles[role] = {}
+        for item in copies:
+            roles[role][item] = submission[item]
+        for name in maps:
+            roles[role][name] = value[name]
+        tags = set(value) - set(maps)
+        params = {
+            'target_group': target_group,
+            'role': role,
+            'group_size': group_size,
+            'sub_id': subid,
+        }
+        for tag in tags:
+            params[tag] = value[tag]
+        roles[role].update({'protocols': {'lava-multinode': params}})
+        subid += 1
+
+    # split the submission based on the roles specified for the actions, retaining order.
+    for role in roles:
+        for action in submission['actions']:
+            for key, value in action.items():
+                if role in value['role']:
+                    actions.setdefault(role, {'actions': []})
+                    actions[role]['actions'].append({copy.deepcopy(key): copy.deepcopy(value)})
+
+    # add other parameters from the lava-multinode protocol
+    for key, value in submission['protocols']['lava-multinode'].iteritems():
+        if key in skip:
+            continue
+        for role in roles:
+            roles[role]['protocols']['lava-multinode'][key] = value
+
+    # set the role for each action to the role of the job instead of the original list..
+    for role in actions:
+        for action in actions[role]['actions']:
+            for key, value in action.items():
+                value['role'] = role
+
+    # jobs dictionary lists the jobs per role,
+    jobs = {}
+    for role in roles:
+        jobs[role] = []
+        for _ in range(0, roles[role]['count']):
+            job = {}
+            job.update(actions[role])
+            job.update(roles[role])
+            del job['count']
+            jobs[role].append(job)
+    return jobs
