@@ -31,6 +31,7 @@ from django.utils.html import escape
 from django.utils.safestring import mark_safe
 from django.db import models
 from django.db.models import Q
+from django_kvstore import models as kvmodels
 
 from django_tables2 import (
     Column,
@@ -60,6 +61,7 @@ from lava_scheduler_app.models import (
     DevicesUnavailableException,
     Worker,
 )
+from lava_scheduler_app import utils
 from dashboard_app.models import BundleStream
 
 from lava.utils.lavatable import LavaTable, LavaView
@@ -1265,6 +1267,9 @@ def _prepare_template(request):
     if request.POST.get("submit_stream"):
         command_submit = copy.deepcopy(COMMAND_SUBMIT_RESULTS)
         command_submit_config = {
+            "SUBMIT_SERVER": "http://{0}{1}RPC2".format(
+                utils.get_fqdn(),
+                reverse('lava.home')),
             "BUNDLE_STREAM": str(request.POST.get("submit_stream"))
         }
         expand_template(command_submit, command_submit_config)
@@ -1319,7 +1324,7 @@ def job_detail(request, pk):
 
     log_file = job.output_file()
     if log_file:
-        with job.output_file() as f:
+        with log_file as f:
             f.seek(0, 2)
             job_file_size = f.tell()
 
@@ -1337,7 +1342,7 @@ def job_detail(request, pk):
         if not job.failure_comment:
             job_errors = getDispatcherErrors(job.output_file())
             if len(job_errors) > 0:
-                msg = "".join(job_errors).strip()
+                msg = job_errors[-1]
                 if msg != "ErrorMessage: None":
                     job.failure_comment = msg
                     job.save()
@@ -1360,6 +1365,10 @@ def job_detail(request, pk):
             'job_file_present': False,
         })
 
+    if "repeat_count" in job.definition:
+        data.update({
+            'expand': True,
+        })
     return render_to_response(
         "lava_scheduler_app/job.html", data, RequestContext(request))
 
@@ -1388,6 +1397,30 @@ def job_definition_plain(request, pk):
     return response
 
 
+@BreadCrumb("Expanded Definition", parent=job_detail, needs=['pk'])
+def expanded_job_definition(request, pk):
+    job = get_restricted_job(request.user, pk)
+    log_file = job.output_file()
+    return render_to_response(
+        "lava_scheduler_app/expanded_job_definition.html",
+        {
+            'job': job,
+            'job_file_present': bool(log_file),
+            'bread_crumb_trail': BreadCrumbTrail.leading_to(expanded_job_definition, pk=pk),
+            'show_cancel': job.can_cancel(request.user),
+            'show_resubmit': job.can_resubmit(request.user),
+        },
+        RequestContext(request))
+
+
+def expanded_job_definition_plain(request, pk):
+    job = get_restricted_job(request.user, pk)
+    response = HttpResponse(job.definition, content_type='text/plain')
+    response['Content-Disposition'] = "attachment; filename=job_%d.json" % \
+        job.id
+    return response
+
+
 @BreadCrumb("Multinode definition", parent=job_detail, needs=['pk'])
 def multinode_job_definition(request, pk):
     job = get_restricted_job(request.user, pk)
@@ -1486,7 +1519,7 @@ def job_log_file(request, pk):
     if not log_file:
         raise Http404
 
-    with job.output_file() as f:
+    with log_file as f:
         f.seek(0, 2)
         job_file_size = f.tell()
 
@@ -1503,7 +1536,7 @@ def job_log_file(request, pk):
             'show_cancel': job.can_cancel(request.user),
             'show_resubmit': job.can_resubmit(request.user),
             'job': TestJob.objects.get(pk=pk),
-            'job_file_present': bool(job.output_file()),
+            'job_file_present': bool(log_file),
             'sections': content,
             'size_warning': size_warning,
             'job_file_size': job_file_size,
@@ -1600,12 +1633,12 @@ def job_cancel(request, pk):
     job = get_restricted_job(request.user, pk)
     if job.can_cancel(request.user):
         if job.is_multinode:
-            multinode_jobs = TestJob.objects.all().filter(
+            multinode_jobs = TestJob.objects.filter(
                 target_group=job.target_group)
             for multinode_job in multinode_jobs:
                 multinode_job.cancel(request.user)
         elif job.is_vmgroup:
-            vmgroup_jobs = TestJob.objects.all().filter(
+            vmgroup_jobs = TestJob.objects.filter(
                 vm_group=job.vm_group)
             for vmgroup_job in vmgroup_jobs:
                 vmgroup_job.cancel(request.user)
@@ -1665,7 +1698,8 @@ def job_resubmit(request, pk):
             else:
                 definition = job.display_definition
 
-            if request.user != job.owner and not request.user.is_superuser:
+            if request.user != job.owner and not request.user.is_superuser \
+               and not utils.is_member(request.user, job.owner):
                 obj = simplejson.loads(definition)
 
                 # Iterate through the objects in the JSON and pop (remove)
@@ -1684,7 +1718,8 @@ def job_resubmit(request, pk):
                                                 break
                 definition = simplejson.dumps(obj, sort_keys=True, indent=4, separators=(',', ': '))
                 response_data["resubmit_warning"] = \
-                    "Since you were not the submitter of the original job,\\nthe bundle stream was removed, please provide a bundle stream."
+                    "The bundle stream was removed because you are neither the submitter "\
+                    "nor in the same group as the submitter. Please provide a bundle stream."
 
             try:
                 response_data["json_input"] = definition
@@ -2254,3 +2289,7 @@ def queue(request):
             'bread_crumb_trail': BreadCrumbTrail.leading_to(queue),
         },
         RequestContext(request))
+
+
+def device_dictionary(request, pk):
+    return "test function"
