@@ -29,6 +29,7 @@ from lava_scheduler_app.models import (
     Device,
     TestJob,
     TemporaryDevice,
+    JSONDataError,
 )
 from lava_scheduler_app import utils
 from lava_scheduler_daemon.worker import WorkerData
@@ -54,16 +55,23 @@ def find_device_for_job(job, device_list):
     If the device has the same tags as the job or all the tags required
     for the job and some others which the job does not explicitly specify,
     check if this device be assigned to this job for this user.
+    Works for pipeline jobs and old-style jobs but refuses to select a
+    non-pipeline device for a pipeline job. Pipeline devices are explicitly
+    allowed to run non-pipeline jobs..
     """
     if job.health_check is True:
         if job.requested_device.status == Device.OFFLINE:
             return job.requested_device
     for device in device_list:
+        if job.is_pipeline and not device.is_pipeline:
+            continue
         if device == job.requested_device:
             if device.can_submit(job.submitter) and\
                     set(job.tags.all()) & set(device.tags.all()) == set(job.tags.all()):
                 return device
     for device in device_list:
+        if job.is_pipeline and not device.is_pipeline:
+            continue
         if device.device_type == job.requested_device_type:
             if device.can_submit(job.submitter) and\
                     set(job.tags.all()) & set(device.tags.all()) == set(job.tags.all()):
@@ -108,18 +116,11 @@ class DatabaseJobSource(object):
 
     def deferForDB(self, func, *args, **kw):
         def wrapper(*args, **kw):
-            # If there is no db connection yet on this thread, create a
-            # connection and immediately commit, because rolling back the
-            # first transaction on a connection loses the effect of
-            # settings.TIME_ZONE when using postgres (see
-            # https://code.djangoproject.com/ticket/17062).
-            transaction.enter_transaction_management()
-            transaction.managed()
+            transaction.set_autocommit(False)
             try:
                 if connection.connection is None:
                     connection.cursor().close()
                     assert connection.connection is not None
-                    transaction.commit()
                 try:
                     return func(*args, **kw)
                 except (DatabaseError, OperationalError, InterfaceError), error:
@@ -137,15 +138,13 @@ class DatabaseJobSource(object):
                             connection.connection = None
                     raise
             finally:
-                # In Django 1.2, the commit_manually() etc decorators only
-                # commit or rollback the transaction if Django thinks there's
-                # been a write to the database.  We don't want to leave
-                # transactions dangling under any circumstances so we
-                # unconditionally issue a rollback.  This might be a teensy
-                # bit wastful, but it wastes a lot less time than figuring out
-                # why your south migration appears to have got stuck...
+                # We don't want to leave transactions dangling under any
+                # circumstances so we unconditionally issue a rollback.  This
+                # might be a teensy bit wastful, but it wastes a lot less time
+                # than figuring out why your south migration appears to have
+                # got stuck...
                 transaction.rollback()
-                transaction.leave_transaction_management()
+                transaction.set_autocommit(True)
         return self.deferToThread(wrapper, *args, **kw)
 
     def _commit_transaction(self, src=None):
@@ -162,12 +161,11 @@ class DatabaseJobSource(object):
         """
         Checks which devices need a health check job and submits the needed
         health checks.
+        Looping is only active once a device is offline.
         """
 
-        for device in Device.objects.all():
-            if device.status != Device.IDLE:
-                continue
-            run_health_check = False
+        for device in Device.objects.filter(
+                Q(status=Device.IDLE) | Q(status=Device.OFFLINE, health_status=Device.HEALTH_LOOPING)):
             if not device.device_type.health_check_job:
                 run_health_check = False
             elif device.health_status == Device.HEALTH_UNKNOWN:
@@ -183,7 +181,11 @@ class DatabaseJobSource(object):
                     datetime.datetime.now() - datetime.timedelta(days=1)
 
             if run_health_check:
-                device.initiate_health_check_job()
+                try:
+                    device.initiate_health_check_job()
+                except JSONDataError:
+                    # already logged, don't allow the daemon to fail.
+                    pass
 
     def _get_job_queue(self):
         """
@@ -198,6 +200,9 @@ class DatabaseJobSource(object):
         we also sort by id to make sure we have a stable order and that jobs
         that came later into the system (as far as the DB is concerned) get
         later into the queue.
+
+        Pipeline jobs are allowed to be assigned but the actual running of
+        a job on a reserved pipeline device is down to the dispatcher-master.
         """
 
         jobs = TestJob.objects.filter(status=TestJob.SUBMITTED)
@@ -215,33 +220,37 @@ class DatabaseJobSource(object):
         using John Doe's private devices over using public devices that could
         be available for other users who don't have their own.
         """
-        devices = Device.objects.filter(status=Device.IDLE)
-        devices = devices.order_by('is_public')
-
+        devices = Device.objects.filter(status=Device.IDLE).order_by('is_public')
         return devices
 
     def _assign_jobs(self):
+        # FIXME: this function needs to be moved to dispatcher-master when lava_scheduler_daemon is disabled.
         jobs = list(self._get_job_queue())
         devices = list(self._get_available_devices())
         for job in jobs:
             device = find_device_for_job(job, devices)
             if device:
-                job.actual_device = device
                 try:
-                    job.submit_token = AuthToken.objects.filter(
-                        user=job.submitter)[0]
-                except IndexError:
-                    job.submit_token = AuthToken.objects.create(
-                        user=job.submitter)
-                device.current_job = job
-                device.state_transition_to(Device.RESERVED, message="Reserved for job %s" % job.display_id)
-                self._commit_transaction(src='%s state' % device.hostname)
-                self.logger.info('%s reserved for job %s', device.hostname,
-                                 job.id)
-                job.save()
-                device.save()
-                if device in devices:
-                    devices.remove(device)
+                    # Make this sequence atomic
+                    with transaction.atomic():
+                        job.actual_device = device
+                        try:
+                            job.submit_token = AuthToken.objects.filter(
+                                user=job.submitter)[0]
+                        except IndexError:
+                            job.submit_token = AuthToken.objects.create(
+                                user=job.submitter)
+                        device.current_job = job
+                        device.state_transition_to(Device.RESERVED, message="Reserved for job %s" % job.display_id)
+                        job.save()
+                        device.save()
+                    if device in devices:
+                        devices.remove(device)
+                    self.logger.info('%s reserved for job %s', device.hostname,
+                                     job.id)
+                except IntegrityError:
+                    # Retry in the next call to _assign_jobs
+                    self.logger.debug("Transaction failed for job %s" % job.display_id)
 
     def _kill_canceling(self, job):
         """
@@ -264,17 +273,23 @@ class DatabaseJobSource(object):
         """
         This method is called in a loop by the scheduler daemon service.
         It's goal is to return a list of jobs that are ready to be started.
+        Note: handles both old and pipeline jobs but only so far as putting
+        devices into a Reserved state. Running pipeline jobs from Reserved
+        is the sole concern of the dispatcher-master.
         """
         self._handle_cancelling_jobs()
 
         if utils.is_master():
+            # FIXME: move into dispatcher-master
             self._submit_health_check_jobs()
             self._assign_jobs()
 
+        # from here on, ignore pipeline jobs.
         my_devices = get_temporary_devices(self.my_devices())
         my_submitted_jobs = TestJob.objects.filter(
             status=TestJob.SUBMITTED,
             actual_device_id__in=my_devices,
+            is_pipeline=False
         )
 
         my_ready_jobs = filter(lambda job: job.is_ready_to_start, my_submitted_jobs)
@@ -315,7 +330,9 @@ class DatabaseJobSource(object):
     def getOutputDirForJobOnBoard_impl(self, board_name):
         device = Device.objects.get(hostname=board_name)
         job = device.current_job
-        return job.output_dir
+        if job:
+            return job.output_dir
+        return None
 
     def getOutputDirForJobOnBoard(self, board_name):
         return self.deferForDB(self.getOutputDirForJobOnBoard_impl, board_name)
@@ -341,12 +358,23 @@ class DatabaseJobSource(object):
     def jobStarted(self, job):
         return self.deferForDB(self.jobStarted_impl, job)
 
-    def jobCompleted_impl(self, board_name, exit_code, kill_reason):
+    def jobCompleted_impl(self, job_id, board_name, exit_code, kill_reason):
+        if not job_id:
+            self.logger.debug('job completion called without a job id on %s',
+                              board_name)
+            return
+        else:
+            job = TestJob.objects.get(id=job_id)
+
         self.logger.debug('marking job as complete on %s', board_name)
         device = Device.objects.get(hostname=board_name)
         old_device_status = device.status
         new_device_status = None
         previous_state = device.previous_state()
+        self.logger.debug('old device status %s, previous device state %s, job state %s' % (
+            Device.STATUS_CHOICES[old_device_status][1],
+            Device.STATUS_CHOICES[previous_state][1],
+            TestJob.STATUS_CHOICES[job.status][1]))
 
         if old_device_status == Device.RUNNING:
             new_device_status = previous_state
@@ -359,8 +387,13 @@ class DatabaseJobSource(object):
                 "Unexpected device state in jobCompleted: %s", device.status)
             new_device_status = Device.IDLE
         if new_device_status is None:
+            self.logger.debug("unhandled old device state")
             new_device_status = Device.IDLE
-        job = device.current_job
+
+        self.logger.debug('new device status %s, previous device state %s, job state %s' % (
+            Device.STATUS_CHOICES[new_device_status][1],
+            Device.STATUS_CHOICES[previous_state][1],
+            TestJob.STATUS_CHOICES[job.status][1]))
 
         # Temporary devices should be marked as RETIRED once the job is
         # complete or canceled.
@@ -371,8 +404,6 @@ class DatabaseJobSource(object):
             except TemporaryDevice.DoesNotExist:
                 self.logger.debug("%s is not a tmp device", device.hostname)
 
-        device.device_version = _get_device_version(job.results_bundle)
-        device.current_job = None
         if job.status == TestJob.RUNNING:
             if exit_code == 0:
                 job.status = TestJob.COMPLETE
@@ -381,23 +412,34 @@ class DatabaseJobSource(object):
         elif job.status == TestJob.CANCELING:
             job.status = TestJob.CANCELED
         else:
-            self.logger.error(
-                "Unexpected job state in jobCompleted: %s", job.status)
-            job.status = TestJob.COMPLETE
+            self.logger.error("Unexpected job state in jobCompleted: %s, probably we are trying job completion for a different job", job.status)
+            return
 
-        msg = "Job %s completed" % job.display_id
-        device.state_transition_to(new_device_status, message=msg, job=job)
-        self._commit_transaction(src='%s state' % device.hostname)
-        self.logger.info('job %s completed on %s', job.id, device.hostname)
+        self.logger.debug('changed job status to %s' % (
+            TestJob.STATUS_CHOICES[job.status][1]))
 
         if job.health_check:
             device.last_health_report_job = job
+            self.logger.debug("old device health status %s" % Device.HEALTH_CHOICES[device.health_status][1])
             if device.health_status != Device.HEALTH_LOOPING:
                 if job.status == TestJob.INCOMPLETE:
                     device.health_status = Device.HEALTH_FAIL
+                    if previous_state not in [Device.OFFLINE, Device.OFFLINING]:
+                        self.logger.debug("taking %s offline, failed health check job %s" % (
+                            device.hostname, job_id))
+                    else:
+                        self.logger.debug("%s is offline, failed health check job %s" % (
+                            device.hostname, job_id))
                     device.put_into_maintenance_mode(None, "Health Check Job Failed")
+                    # update the local variable to track the effect of the external function call
+                    new_device_status = device.status
+                    if new_device_status == Device.OFFLINING:
+                        new_device_status = Device.OFFLINE  # offlining job is complete.
                 elif job.status == TestJob.COMPLETE:
                     device.health_status = Device.HEALTH_PASS
+                    if old_device_status == Device.RUNNING:
+                        new_device_status = Device.IDLE
+            self.logger.debug("new device health status %s" % Device.HEALTH_CHOICES[device.health_status][1])
 
         bundle_file = os.path.join(job.output_dir, 'result-bundle')
         if os.path.exists(bundle_file):
@@ -412,12 +454,26 @@ class DatabaseJobSource(object):
             else:
                 job._results_bundle = bundle
 
+        self.logger.debug('new device status %s, previous device state %s, job state %s' % (
+            Device.STATUS_CHOICES[new_device_status][1],
+            Device.STATUS_CHOICES[previous_state][1],
+            TestJob.STATUS_CHOICES[job.status][1]))
+
         job.end_time = datetime.datetime.utcnow()
-        token = job.submit_token
+
         job.submit_token = None
+
+        device.device_version = _get_device_version(job.results_bundle)
+        device.current_job = None
+        msg = "Job %s completed" % job.display_id
+        device.state_transition_to(new_device_status, message=msg, job=job)
+        self._commit_transaction(src='%s state' % device.hostname)
+
         device.save()
         job.save()
         self._commit_transaction(src='jobCompleted_impl')
+        self.logger.info('job %s completed on %s', job.id, device.hostname)
+
         if utils.is_master():
             try:
                 job.send_summary_mails()
@@ -430,8 +486,9 @@ class DatabaseJobSource(object):
             worker = WorkerData()
             worker.notify_on_incomplete(job.id)
 
-    def jobCompleted(self, board_name, exit_code, kill_reason):
-        return self.deferForDB(self.jobCompleted_impl, board_name, exit_code, kill_reason)
+    def jobCompleted(self, job_id, board_name, exit_code, kill_reason):
+        return self.deferForDB(self.jobCompleted_impl, job_id, board_name,
+                               exit_code, kill_reason)
 
     def jobCheckForCancellation_impl(self, board_name):
         device = Device.objects.get(hostname=board_name)
@@ -442,7 +499,7 @@ class DatabaseJobSource(object):
         return self.deferForDB(self.jobCheckForCancellation_impl, board_name)
 
     def _handle_cancelling_jobs(self):
-        cancel_list = TestJob.objects.all().filter(status=TestJob.CANCELING)
+        cancel_list = TestJob.objects.filter(status=TestJob.CANCELING, is_pipeline=False)
         # Pick up TestJob objects in Canceling and ensure that the cancel completes.
         # call _kill_canceling to terminate any lava-dispatch calls
         # Explicitly set a DeviceStatusTransition as jobs which are stuck in Canceling
