@@ -67,7 +67,6 @@ MIDDLEWARE_CLASSES = [
     'django.contrib.sessions.middleware.SessionMiddleware',
     'django.contrib.auth.middleware.AuthenticationMiddleware',
     'django.middleware.common.CommonMiddleware',
-    'django.middleware.transaction.TransactionMiddleware',
     'django.contrib.messages.middleware.MessageMiddleware',
 ]
 
@@ -159,28 +158,10 @@ AUTHENTICATION_BACKENDS = (
     # Atlassian Crowd auth.
     # 'crowdrest.backend.CrowdRestBackend',
 
-    # Uncomment LDAPBackend and comment OpenIDBackend to enable LDAP auth and
-    # make sure LDAP auth parameters are set properly.
-    # 'django_auth_ldap.backend.LDAPBackend',
-
     'django_openid_auth.auth.OpenIDBackend',
     'django.contrib.auth.backends.ModelBackend',
 )
 
-# Uncomment the following lines to configure LDAP auth parameters. For details
-# on configuration parameters,
-# see: https://pythonhosted.org/django-auth-ldap/authentication.html
-# AUTH_LDAP_SERVER_URI = "ldap://yourldapservername"
-# AUTH_LDAP_BIND_DN = ""
-# AUTH_LDAP_BIND_PASSWORD = ""
-# AUTH_LDAP_USER_DN_TEMPLATE = "uid=%(user)s,ou=users,dc=example,dc=com"
-
-# Uncomment the following lines, if you need logging enabled for LDAP auth.
-# import logging
-# logger = logging.getLogger('django_auth_ldap')
-# logger.addHandler(logging.StreamHandler())
-# logger.setLevel(logging.DEBUG)
-
 OPENID_CREATE_USERS = True
 OPENID_LAUNCHPAD_TEAMS_MAPPING_AUTO = False
 OPENID_UPDATE_DETAILS_FROM_SREG = True
@@ -203,3 +184,53 @@ ME_PAGE_ACTIONS = [
 ALLOWED_HOSTS = ['*']
 
 SESSION_SERIALIZER = 'django.contrib.sessions.serializers.PickleSerializer'
+
+# this is a tad ugly but the upstream package still needs something here.
+KEY_VALUE_STORE_BACKEND = 'db://lava_scheduler_app_devicedictionarytable'
+
+# Logging
+
+LOGGING = {
+    'version': 1,
+    'disable_existing_loggers': False,
+    'filters': {
+        'require_debug_false': {
+            '()': 'django.utils.log.RequireDebugFalse'
+        }
+    },
+    'formatters': {
+        'lava': {
+            'format': '%(levelname)s %(asctime)s %(module)s %(message)s'
+        }
+    },
+    'handlers': {
+        'logfile': {
+            'class': 'logging.handlers.WatchedFileHandler',
+            'filename': '/var/log/lava-server/django.log',
+            'formatter': 'lava'
+        }
+    },
+    'loggers': {
+        'django': {
+            'handlers': ['logfile'],
+            # DEBUG outputs all SQL statements
+            'level': 'ERROR',
+            'propagate': True,
+        },
+        'django_auth_ldap': {
+            'handlers': ['logfile'],
+            'level': 'INFO',
+            'propagate': True,
+        },
+        'dashboard_app': {
+            'handlers': ['logfile'],
+            'level': 'INFO',
+            'propagate': True,
+        },
+        'lava_scheduler_app': {
+            'handlers': ['logfile'],
+            'level': 'INFO',
+            'propagate': True,
+        }
+    }
+}
