@@ -47,7 +47,7 @@ ADMIN_MEDIA_PREFIX = distro_settings.ADMIN_MEDIA_PREFIX
 
 # List of absolute pathnames used to resolve templates.
 TEMPLATE_DIRS = [os.path.join(os.path.dirname(__file__), '..', 'templates')]
-TEMPLATE_DIRS.extend(distro_settings.TEMPLATE_DIRS)
+TEMPLATE_DIRS = distro_settings.TEMPLATE_DIRS + TEMPLATE_DIRS
 
 # Like TEMPLATE_DIRS but for static files
 STATICFILES_DIRS = distro_settings.STATICFILES_DIRS
@@ -145,11 +145,8 @@ if AUTH_LDAP_SERVER_URI:
         AUTH_LDAP_USER_FLAGS_BY_GROUP = distro_settings.get_setting(
             "AUTH_LDAP_USER_FLAGS_BY_GROUP")
 
-    # Enable logging for LDAP auth
-    import logging
-    logger = logging.getLogger('django_auth_ldap')
-    logger.addHandler(logging.StreamHandler())
-    logger.setLevel(logging.DEBUG)
+    # read any LDAP login message to use from the settings.conf
+    LOGIN_MESSAGE_LDAP = distro_settings.get_setting("LOGIN_MESSAGE_LDAP", "")
 
 # Load extensions
 loader.contribute_to_settings(locals(), distro_settings)
